unit Model.ZLibDemo;

// Define TIOPF directive if tiOPF installed (www.tiopf.com)
{.$DEFINE TIOPF}

interface
uses
  SysUtils
 ,Classes
 ,System.ZLib
 ;

type
  TLogMsgEvent = procedure ( const AMsg : string ) of object;

  TZlibDemo = class
  private
    fFrequency         : Int64;
    fOnLogMsg          : TLogMsgEvent;
    fCompress          : boolean;
    fCompressClassName : String;
    fEncryptType       : string;
    procedure LogMsg(AMsg: string);
    procedure SetCompress(const Value: boolean);
    procedure DisplayStreamHex(const AStream: TStream; const AMsg : String = ''; const ALength : Integer = 40);
  public
    constructor Create;
    property OnLogMsg    : TLogMsgEvent   read fOnLogMsg    write fOnLogMsg;
    property Compress    : boolean        read fCompress    write SetCompress;
    property EncryptType : string         read fEncryptType write fEncryptType;
{$IFDEF TIOPF}
    procedure ExecuteTiOPF(AFileName: TFileName);
{$ENDIF}
    procedure ExecuteDirect(AFileName: TFileName; const ALevel: TZCompressionLevel);
  end;
implementation
uses
  Windows
{$IFDEF TIOPF}
 ,tiCompress
 ,tiCompressNone
 ,tiCompressZLib
 ,tiConstants
 ,tiEncrypt
 ,tiEncryptNone
 ,tiEncryptSimple
 ,tiEncryptBlockCipher
 ,tiEncryptBlowfish
 ,tiEncryptDES
{$ENDIF}
 ;

{ TZlibDemo }

constructor TZlibDemo.Create;
begin
  inherited Create;
  QueryPerformanceFrequency(fFrequency);
  Compress := True;
end;

procedure TZlibDemo.LogMsg(AMsg: string);
begin
  if Assigned( fOnLogMsg ) then
    fOnLogMsg( AMsg );
end;


procedure TZlibDemo.SetCompress(const Value: boolean);
begin
  fCompress := Value;
{$IFDEF TIOPF}
  if fCompress then
    fCompressClassName := cgsCompressZLib
  else
    fCompressClassName := cgsCompressNone;
{$ENDIF}
end;

procedure TZlibDemo.DisplayStreamHex(const AStream: TStream; const AMsg : String = ''; const ALength: Integer = 40);
// Display a few hex chars. (Not implemented)
var
  b       : Byte;
  lBuffer : TBytes;
  lStrHex : String;
begin
  SetLength(lBuffer, ALength);
  AStream.Position := 0;
  AStream.Read(lBuffer, Length(lBuffer));
  lStrHex := '';

  for b in lBuffer do
    lStrHex := lStrHex + b.ToHexString;

  LogMsg( AMsg + ': ' + StringOf(lBuffer) + ' ' + lStrHex );
  AStream.Position := 0;
end;

{$IFDEF TIOPF}
procedure TZlibDemo.ExecuteTiOPF(AFileName: TFileName);
var
  lStreamFrom,
  lStreamTo,
  lStreamEncrypt   : TMemoryStream;
  lStartTime,
  lEndTime         : Int64;
  lFileNameOut,
  lFileNameEncrypt : TFilename;
  lEncrypt         : TtiEncryptAbs;
begin
  LogMsg( '' );
  LogMsg( '----------------------------------------' );
  LogMsg( 'Processing (via tiOPF): ' + AFileName);

  lStreamFrom    := TMemoryStream.Create;
  lStreamTo      := TMemoryStream.Create;
  lStreamEncrypt := TMemoryStream.Create;
  try
    lStreamFrom.LoadFromFile(AFileName);
//    DisplayStreamHex(lStreamFrom, 'From');

// Compress...
    QueryPerformanceCounter(lStartTime);
    tiCompressStream(lStreamFrom, lStreamTo, fCompressClassName);
    QueryPerformanceCounter(lEndTime);

//    DisplayStreamHex(lStreamTo, 'To  ');
    LogMsg( '' );
    if lStreamFrom.Size > 0 then
      LogMsg(Format('%s FROM: %f3.3 Kb  TO: %f3.3 Kb  RATIO: %f%%   TIME: %s mSec',
                   [fCompressClassName, lStreamFrom.Size / 1000, lStreamTo.Size / 1000, ((lStreamTo.Size * 100) / lStreamFrom.Size) , FloatToStrF(((lEndTime - lStartTime) * 1000 / fFrequency), ffNumber, 7, 2)]));

    lFileNameOut     := AFileName + '.out';
    lStreamTo.SaveToFile(lFileNameOut);
    LogMsg( 'Saved to: ' + ExtractFileName(lFileNameOut) );

    lEncrypt         := gEncryptFactory.CreateInstance(fEncryptType);
    lEncrypt.EncryptStream(lStreamTo,lStreamEncrypt);
    lFileNameEncrypt := lFileNameOut + '.crypt';
    lStreamEncrypt.SaveToFile(lFileNameEncrypt);
    LogMsg( 'Encrypt saved to: ' + ExtractFileName(lFileNameEncrypt) );
    LogMsg( '' );

  finally
    lStreamFrom.Free;
    lStreamTo.Free;
    lStreamEncrypt.Free;
  end;
end;
{$ENDIF}

procedure TZlibDemo.ExecuteDirect(AFileName: TFileName; const ALevel: TZCompressionLevel);
var
  lStreamFrom,
  lStreamTo,
  lStreamBack  : TMemoryStream;
  lStartTime,
  lEndTime     : Int64;
  lFileNameOut,
  lFileNameBack: TFilename;
begin
  LogMsg( '' );
  LogMsg( '----------------------------------------' );
  LogMsg( 'Processing (via zlib): ' + AFileName);

  lStreamFrom    := TMemoryStream.Create;
  lStreamTo      := TMemoryStream.Create;
  lStreamBack    := TMemoryStream.Create;
  try
    lStreamFrom.LoadFromFile(AFileName);
//    DisplayStreamHex(lStreamFrom, 'From');

// Compress...

    QueryPerformanceCounter(lStartTime);
    ZCompressStream(lStreamFrom, lStreamTo, ALevel);
    QueryPerformanceCounter(lEndTime);

//    DisplayStreamHex(lStreamTo, 'To  ');
    LogMsg( '' );
    if lStreamFrom.Size > 0 then
      LogMsg(Format('%s FROM: %f3.3 Kb  TO: %f3.3 Kb  RATIO: %f%%   TIME: %s mSec',
                   [fCompressClassName, lStreamFrom.Size / 1000, lStreamTo.Size / 1000, ((lStreamTo.Size * 100) / lStreamFrom.Size) , FloatToStrF(((lEndTime - lStartTime) * 1000 / fFrequency), ffNumber, 7, 2)]));

    lFileNameOut     := AFileName + '.out';
    lStreamTo.SaveToFile(lFileNameOut);
    LogMsg( 'Saved to: ' + ExtractFileName(lFileNameOut) );



// De-Compress...

    lStreamTo.Position := 0;
    QueryPerformanceCounter(lStartTime);
    ZDecompressStream(lStreamTo, lStreamBack);
    QueryPerformanceCounter(lEndTime);

    LogMsg( '' );
    LogMsg('Decompress TIME: ' + FloatToStrF(((lEndTime - lStartTime) * 1000 / fFrequency), ffNumber, 7, 2) + ' mSec');
    lFileNameBack    := lFileNameOut + '.dup';
    lStreamBack.SaveToFile(lFileNameBack);
    LogMsg( 'Restored to: ' + ExtractFileName(lFileNameBack) );

    LogMsg( '' );

  finally
    lStreamFrom.Free;
    lStreamTo.Free;
    lStreamBack.Free;
  end;
end;

end.
